/*---------------------------------------------------------------------------/

getdata.c:	module to get data for locations, objects etc. into main
		memory from auxiliary files.
started:	21:11:1987

/---------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "main.h"
#include "testing.h"
#include "getdata.h"
#include "aux.h"

/*--------------------------------------------------------------------------*/

FILE *fp;

/*--------------------------------------------------------------------------*/

char *malloc ();
char *realloc ();

/*--------------------------------------------------------------------------*/

int getnumber ()
{
  int retval = 0;
  fscanf (fp, "%d", &retval);
  return (retval);
}

/*--------------------------------------------------------------------------*/

void getdescription (description)
  char description [DESCLEN];
{
  char line [LINELEN];
  *description = '\0';
  fgets (line, LINELEN, fp);
  while (*line != '@')
  {
    if (*line != '#')
      strcpy (description, strcat (description, line));
    fgets (line, LINELEN, fp);
  }
  description [strlen (description) -1] = '\0';
}

/*--------------------------------------------------------------------------*/

void getname (name)
  char name [NAMELEN];
{
  *name = '#';
  while (*name == '#')
  {
    fgets (name, NAMELEN, fp);
  }
  name [strlen (name) -1] = '\0';
}

/*--------------------------------------------------------------------------*/

int getmeaning (xmeaning)
  char xmeaning [NAMELEN];
{
  char ch;
  int index = 0;
  while (isalpha (ch = getc (fp)))
    xmeaning [index++] = ch;
  xmeaning [index] = '\0';
  return (ch == ' ');
}

/*--------------------------------------------------------------------------*/

int getlocations ()
{
  char description [DESCLEN];
  char name [NAMELEN];
  char ch;
  int i, j;
  if (fp = fopen (LOCATIONS, "r"))
  {
    locations = (location**) malloc (sizeof (location*));
    i = 1;
    ch = getc (fp);
    while (!feof (fp))
    {
      locations = (location**) realloc ((char*) locations, (unsigned) (i+1)*sizeof (location*));
      locations [i] = (location*) malloc (sizeof (location));
      ungetc (ch, fp);
      getdescription (description);
      getname (name);
      for (j=1; j <= NO_DIRECTIONS; j++)
        locations [i]->exits [j] = getnumber ();
      fscanf (fp, "%*s%*c");
      locations [i]->description = malloc ((unsigned) strlen (description)+1);
      strcpy (locations [i]->description, description);
      locations [i]->name = malloc ((unsigned) strlen (name)+1);
      strcpy (locations [i]->name, name);
      locations [i]->seen = 0;
      i++;;
      ch = getc (fp);
    }
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: can't open %s.\n", basename (argv[0]), LOCATIONS);
    exit (1);
  }
  return (i-1);
}

/*--------------------------------------------------------------------------*/

int getobjects ()
{
  char ch;
  char description [DESCLEN];
  char current [WORDLEN];
  int i = 1, j, eoln;
  if (fp = fopen (OBJECTS, "r"))
  {
    objects = (object**) malloc (sizeof (object*));
    ch = getc (fp);
    while (!feof (fp))
    {
      ungetc (ch, fp);
      objects = (object**) realloc ((char*) objects, (unsigned) (i+1)*sizeof (object*));
      objects [i] = (object*) malloc (sizeof (object));
      getname (description);
      objects [i]->description = malloc ((unsigned) strlen (description)+1);
      strcpy (objects [i]->description, description);

      j = 1;
      do
      {
        eoln = !getmeaning (current);
        strcpy (objects [i]->adj.wordv [j], current);
        j++;
      } while (!eoln);
      objects [i]->adj.wordc = --j;

      j = 1;
      do
      {
        eoln = !getmeaning (current);
        strcpy (objects [i]->name.wordv [j], current);
        j++;
      } while (!eoln);
      objects [i]->name.wordc = --j;

      objects [i]->location = getnumber ();
      objects [i]->value = getnumber ();
      objects [i]->weight = getnumber ();
      objects [i]->size = getnumber ();
      objects [i]->flag = getnumber ();
      objects [i]->container = getnumber ();
      objects [i]->seen = getnumber ();
      fscanf (fp, "%*s%*c");
      i++;;
      ch = getc (fp);
    }
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: can't open %s.\n", basename (argv[0]), OBJECTS);
    exit (1);
  }
  return (i-1);
}

/*--------------------------------------------------------------------------*/

int getmeanings ()
{
  char ch;
  int i, j, eoln;
  char current [WORDLEN];
  if (fp = fopen (MEANINGS, "r"))
  {
    meanings = (meaning**) malloc (sizeof (meaning*));
    i = 1;
    ch = getc (fp);
    while (!feof (fp))
    {
      ungetc (ch, fp);
      meanings = (meaning**) realloc ((char*) meanings, (unsigned) (i+1)*sizeof (meaning*));
      meanings [i] = (meaning*) malloc (sizeof (meaning));
      j = 1;
      do
      {
        eoln = !getmeaning (current);
        strcpy (meanings [i]->wordv [j], current);
        j++;
      }
      while (!eoln);
      meanings [i]->wordc = --j;

   /* printf ("Meaning %d: ", i);
      for (j=1; j <= meanings [i]->wordc; j++)
        printf ("%s, ", meanings [i]->wordv [j]);
      printf ("%c%c.\n", DELETE, DELETE); */

      i++;
      ch = getc (fp);
    }
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: Can't open %s.\n", basename (argv[0]), MEANINGS);
    exit (1);
  }
  return (i-1);
}

/*--------------------------------------------------------------------------*/

int getdirections ()
{
  int i, eoln;
  char current [WORDLEN];
  if (fp = fopen (DIRECTIONS, "r"))
  {
    directions = (direction**) malloc ((unsigned) sizeof (direction*));
    /*printf ("directions = %d\n", directions);*/
    i=1;
    do
    {
      directions = (direction**) realloc ((direction**) directions, (unsigned) (i+1)*sizeof (direction*));
      eoln = !getmeaning (current);
      /*fprintf (stderr, "\ni, current = %d, \"%s\".\n", i, current);*/
      /*fprintf (stderr, "directions [i] = \"%d\".\n", directions [i]);*/
      directions [i] = (direction*) malloc (sizeof (direction));
      /*fprintf (stderr, "directions [i] = \"%d\".\n", directions [i]);*/
      directions [i]->description = malloc ((unsigned) (strlen (current)+1));
      /*fprintf (stderr, "directions [i] = \"%d\".\n", directions [i]);*/
      strcpy (directions [i]->description, current);
      /*fprintf (stderr, "directions [i] = \"%d\".\n", directions [i]);*/
      *current = tolower (*current);
      directions [i]->name = malloc ((unsigned) (strlen (current)+1));
      /*fprintf (stderr, "directions [i] = \"%d\".\n", directions [i]);*/
/*
strcpy (directions [i]->name, current);
*/
      current [1] = '\0';
      strcpy (directions [i]->abbrev, current);
      i++;
    } while (!eoln);
    printf ("%d\n",i);
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: Can't open %s.\n", basename (argv[0]), DIRECTIONS);
    exit (1);
  }
  return (--i);
}

/*--------------------------------------------------------------------------*/

int getarticles ()
{
  int j, eoln;
  char current [WORDLEN];
  if (fp = fopen (ARTICLES, "r"))
  {
    j = 1;
    do
    {
      eoln = !getmeaning (current);
      strcpy (articles.wordv [j], current);
      j++;
    } while (!eoln);
    articles.wordc = --j;
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: Can't open %s.\n", basename (argv[0]), ARTICLES);
    exit (1);
  }
  return (j);
}


/*--------------------------------------------------------------------------*/

int getprepositions ()
{
  int j, eoln;
  char current [WORDLEN];
  if (fp = fopen (PREPOSITIONS, "r"))
  {
    j = 1;
    do
    {
      eoln = !getmeaning (current);
      strcpy (prepositions.wordv [j], current);
      j++;
    } while (!eoln);
    prepositions.wordc = --j;
    fclose (fp);
  }
  else
  {
    fprintf (stderr, "%s: Can't open %s.\n", basename (argv[0]), PREPOSITIONS);
    exit (1);
  }
  return (j);
}

/*--------------------------------------------------------------------------*/

void initialise ()
{
  no_locations = getlocations ();
  no_objects = getobjects ();
  no_meanings = getmeanings ();
  no_directions = getdirections ();
  no_articles = getarticles ();
  no_prepositions = getprepositions ();
}

/*--------------------------------------------------------------------------*/
