/*---------------------------------------------------------------------------/

look.c:		Module to print location descriptions, inventory etc.
started:	23:11:1987

/---------------------------------------------------------------------------*/

#include <stdio.h>
#include "main.h"
#include "look.h"
#include "lex.h"
#include "y.tab.h"
#include "aux.h"
#include "parse.h"

/*--------------------------------------------------------------------------*/

#define PLURAL(QUANTITY) (QUANTITY)!=1?'s':'\0'

/*--------------------------------------------------------------------------*/

void describe_text (here)
  int here;
{
  if (mode & 2 || mode & 1 && !locations [here]->seen)
    printf ("%s\n", locations [here]->description);
  else
    printf ("%s\n", locations [here]->name);
  locations [here]->seen=1;
}

/*--------------------------------------------------------------------------*/

int nonempty (here)
  int here;
{
  int i, retval=0;
  for (i=1; i<=no_objects; i++)
    if (objects [i]->location == here)
      retval=1;
  return (retval);
}

/*--------------------------------------------------------------------------*/

void indent (level)
  int level;
{
  int i;
  for (i=1; i<=INDENT*level; i++)
    printf (" ");
}

/*--------------------------------------------------------------------------*/

void printobject (index, level)
  int index;
  int level;
{
  int i;
  indent (level);
  printf ("A %s", objects [index]->description);
  if (objects [index]->container & 11)
  {
    if (nonempty (-index))
    {
      if (objects [index]->container & 8)
        printf (" on which you can see:\n");
      else
        if (objects [index]->container & 2)
          printf (" which is open and contains:\n");
        else
          printf (" which contains:\n");
      for (i=1; i <= no_objects; i++)
        if (objects [i]->location == -index)
          printobject (i, level+1);
    }
    else
      if (objects [index]->container & 2)
        printf (" which is open.\n");
      else
        printf (".\n");
  }
  else
    printf (".\n");
}

/*--------------------------------------------------------------------------*/

void describe_objects (here)
  int here;
{
  int index;
  if (mode & 8 && nonempty (here))
  {
    printf ("You can also see:\n");
    for (index=1; index<=no_objects; index++)
      if (objects [index]->location == here)
        printobject (index, 1);
  }
}

/*--------------------------------------------------------------------------*/

void describe_exits (here)
  int here;
{
  int i;
  int no_exits = 0;
  int no_sofar = 0;
  if (mode & 4)
  {
    for (i=1; i <= NO_DIRECTIONS; i++)
      if (locations [here]->exits [i])
        no_exits++;
    if (no_exits)
    {
      if (no_exits == 1)
        printf ("The only obvious exit leads ");
      else
        printf ("Obvious exits lead ");
      for (i=1; i <= NO_DIRECTIONS; i++)
        if (locations [here]->exits [i])
        {
          no_sofar++;
          if (no_sofar == no_exits && no_exits > 1)
            printf ("%c%c and ", DELETE, DELETE);
          printf ("%s, ", directions [i]->description);
        }
      printf ("%c%c.\n", DELETE, DELETE);
    }
    else
      printf ("There seems to be no way out of here.\n");
  }
}

/*--------------------------------------------------------------------------*/

void describe (here)
  int here;
{
  printf ("\n");
  describe_text (here);
  describe_objects (here);
  describe_exits (here);
}

/*--------------------------------------------------------------------------*/

int generalhere (item, mask, xlocation)
  int item;
{
  int retval = 0;
  if (objects [item]->location == xlocation)
    retval = 1;
  else
    if (objects [item]->location<0 && objects [-objects [item]->location]->container & mask)
      retval = generalhere (-objects [item]->location, mask, xlocation);
  return (retval);
}

/*--------------------------------------------------------------------------*/

int accessheld (item)
  int item;
{
  return (generalhere (item, 10, 0));
}

/*--------------------------------------------------------------------------*/

int accesshere (item)
  int item;
{
  return (generalhere (item, 10, whereami));
}

/*--------------------------------------------------------------------------*/

int visiheld (item)
  int item;
{
  return (generalhere (item, 11, 0));
}

/*--------------------------------------------------------------------------*/

int visihere (item)
  int item;
{
  return (generalhere (item, 11, whereami));
}

/*--------------------------------------------------------------------------*/

int accessible (item)
  int item;
{
  return (accesshere (item) || accessheld (item));
}

/*--------------------------------------------------------------------------*/

int visible (item)
  int item;
{
  return (visihere (item) || visiheld (item));
}

/*--------------------------------------------------------------------------*/

int totalweight (item)
  int item;
{
  int i;
  int retval = 0;
  retval = objects [item]->weight;
  if (objects [item]->container)
    for (i=1; i<=no_objects; i++)
      if (objects [i]->location == -item)
        retval += totalweight (i);
  return (retval);
}

/*--------------------------------------------------------------------------*/

int printscore ()
{
  printf ("\nIn %d move%c, ", no_moves, PLURAL(no_moves));
  printf ("you have scored %d point%c.\n", score, PLURAL(score));
  printf ("Your stamina currently stands at %d point%c.\n", stamina, PLURAL(stamina));
  return (0);
}

/*--------------------------------------------------------------------------*/

int inventory ()
{
  int i, gotone=0;
  for (i=1; i<=no_objects; i++)
    if (!objects [i]->location)
    {
      if (!gotone)
	printf ("\nYou are carrying:\n");
      printobject (i, 1);
      gotone=1;
    }
  if (gotone)
    printf ("The total weight of this is %d pound%c.\n", load, PLURAL(load));
  else
    printf ("\nYou are not carrying anything.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int verbose ()
{
  mode|=2;
  printf ("Maximum verbosity.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int brief ()
{
  mode|=1;
  mode&=13;
  printf ("Brief descriptions.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int superbrief ()
{
  mode&=12;
  printf ("Superbrief descriptions.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int exits ()
{
  mode|=4;
  printf ("Exits will be explicitly listed.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int noexits ()
{
  mode&=11;
  printf ("Exits will not be explicitly listed.\n");
  return (0);
}

/*--------------------------------------------------------------------------*/

int isconjunction (word)
  char *word;
{
  return (!strcmp (word, "and"));
}

/*--------------------------------------------------------------------------*/

int isnoun (word)
  char *word;
{
  int i, j, retval = 0;
  for (i=1; i<=no_objects; i++)
  {
    for (j=1; j<=objects [i]->name.wordc; j++)
      if (!strcmp (word, objects [i]->name.wordv [j]))
      {
	retval = 1;
	j = objects [i]->name.wordc;
	i = no_objects;
      }
  }
  return (retval);
}
  
/*--------------------------------------------------------------------------*/

int isadjective (word)
  char *word;
{
  int i, j, retval = 0;
  for (i=1; i<=no_objects; i++)
  {
    for (j=1; j<=objects [i]->adj.wordc; j++)
      if (!strcmp (word, objects [i]->adj.wordv [j]))
      {
	retval = 1;
	j = objects [i]->adj.wordc;
	i = no_objects;
      }
  }
  return (retval);
}
  
/*--------------------------------------------------------------------------*/

int isverb (word)
  char *word;
{
  int i, j, retval = 0;
  for (i=1; i<=no_meanings; i++)
    for (j=1; j<=meanings [i]->wordc; j++)
      if (!strcmp (word, meanings [i]->wordv [j]))
      {
	retval = 1;
	j = meanings [i]->wordc;
	i = no_meanings;
      }
  return (retval);
}

/*--------------------------------------------------------------------------*/

int isarticle (word)
  char *word;
{
  int j, retval = 0;
  for (j=1; j<=articles.wordc; j++)
    if (!strcmp (word, articles.wordv [j]))
      retval = 1;
  return (retval);
}

/*--------------------------------------------------------------------------*/

int ispreposition (word)
  char *word;
{
  int j, retval = 0;
  for (j=1; j<=prepositions.wordc; j++)
    if (!strcmp (word, prepositions.wordv [j]))
      retval = 1;
  return (retval);
}

/*--------------------------------------------------------------------------*/

int isdirection (word)
  char *word;
{
  int j, retval = 0;
  for (j=1; j<=no_directions; j++)
    if ((!strcmp (word, directions [j]->description))                              || (!strcmp (word, directions [j]->name))                                      || (!strcmp (word, directions [j]->abbrev)))
      retval = 1;
  return (retval);
}

/*--------------------------------------------------------------------------*/

int isscenery (word)
  char *word;
{
  return ((instring (word, locations [whereami]->description)) != NULL);
}

/*--------------------------------------------------------------------------*/

int whatword (oldword)
  char *oldword;
{
  int symboltype = UNKNOWNWORD;
  char *word;
  char *malloc ();
  word = malloc (strlen (oldword));
  lowerword (word, oldword);
  if (isconjunction (word))
    symboltype = CONJUNCTION;
  else
    if (isarticle (word))
      symboltype = ARTICLE;
    else
      if (ispreposition (word))
	symboltype = PREPOSITION;
      else
	if (isverb (word))
	  symboltype = VERB;
	else
	  if (isdirection (word))
	    symboltype = DIRECTION;
	  else
	    if (isadjective (word))
	      symboltype = ADJECTIVE;
	    else
	      if (isnoun (word))
		symboltype = NOUN;
	      else
		if (isscenery (word))
		  symboltype = SCENERY;
  return (symboltype);
}
  
/*--------------------------------------------------------------------------*/

int whattype (word)
  char *word;
{
  int symboltype = UNKNOWNWORD;
  if (*word == ',')
    symboltype = CONJUNCTION;
  else
    if (*word == '.' || *word == '\n')
      symboltype = NOMORE;
    else
      if (*word == '!')
	symboltype = METACOMMAND;
      else
	symboltype = whatword (word);
  return (symboltype);
}
  
/*--------------------------------------------------------------------------*/
